#!/bin/perl
# -- configuration -----------

$spam_file = "\/nospam\.html";
$spam_threshold = 10;

#
# chk_spammer.pl
# find accesses to $spam_file (nospam.html) and add to deny list
# when the count of the access exceeds the $spam_threshold (10),
# see the configuration lines below
# 
# options:
#   STDIN: source .htaccess
#   STDOUT: result .htaccess
#   param: access log file
#   
# typical usage:
#  echo | perl chk_spammer.pl access_log > .htaccess.new
#  cat .htaccess | perl chk_spammer.pl access_log > .htaccess.new
# 
# Licensed under GPLv3 
# by Taka Matsutsuka, Mar 2015
# http://www.markn.org/blog/
#

use Time::Local;

# ----------------------------

# Variables
#
# @input_file : exsiting htaccess
# this script assumes the latter half of the htaccess is generated
# by this script
#
# %count : (address, count)
#

# Define the access_log file
$access_log = $ARGV[0];
if ($access_log eq "") {
    $access_log = "access_log";
}

%monthnames = ( 
    jan => 0, feb => 1, mar => 2, apr => 3, may =>  4,  jun =>  5,
    jul => 6, aug => 7, sep => 8, oct => 9, nov => 10,  dec => 11);



if ($#ARGV < 0) {
    print "Usage chk_spammer.pl <combined logfile>\n";
    exit;
}

# Skip to generated lines
$have_input = 0;
while (<STDIN>) {
    if (/Generated by chk_spammer.pl/) {
	$have_input = 1;
	last;
    }
    push (@input_file, $_);
}

# Read existing htaccess deny entries
if ($have_input) {
    $time = <STDIN>;

    ($day, $month, $year, $hour, $min, $sec, $tz)
	= ($time =~ /(\d{2}).(\w{3}).(\d{4}).(\d{2}).(\d{2}).(\d{2}) ([\+\-]\d{4})/);
    $lastUpdated = timelocal($sec, $min, $hour, $day, $monthnames{lc($month)}, $year);
    while (<STDIN>) {
	if (/deny from ([0-9a-zA-z\-\.]+)[^\d]+(\d+) spam access/) {
	    $count{$1} = $2;
	}
    }
}

# Read access_log file
if (!open(LOG, "< $access_log")) {
    die ("cannot open $access_log");
}

$skipped = 0;
$spams = 0;
$lines = 0;

while (<LOG>) {
    ($host, $ident, $user, $time, $request, $status, $bytes, $referer, $agent)
	= ($_ =~ /^([^ ]*) ([^ ]*) ([^ ]*) \[([^]]*)\] "(.*?)" ([^ ]*) ([^ ]*) "(.*?)" "(.*?)"/); 

    ($day, $month, $year, $hour, $min, $sec, $tz)
	= ($time =~ /^(\d{2}).(\w{3}).(\d{4}).(\d{2}).(\d{2}).(\d{2}) ([\+\-]\d{4})/);
    $current = timelocal($sec, $min, $hour, $day, $monthnames{lc($month)}, $year);
    if ($current <= $lastUpdated) {
	$skipped++;
    } else {
	$lines++;
	if ($request =~ /$spam_file/) {
	    if ($host =~ /[0-9a-zA-Z\-]+\.[0-9a-zA-Z\-]+/) {
	        $spams++;
	        $count{$host}++;
	    }
	}
    }
}

# Emit non-editing entries of htaccess (former half of htaccess)
foreach $i (@input_file) {
    print $i;
}

# Emit generating entries of htaccess
print "# Generated by chk_spammer.pl\n";
print "# Last updated: $time\n";
$out1 = 0;
$out2 = 0;
foreach (sort {$count{$b} <=> $count{$a}} keys %count) {
    if ($count{$_} >= $spam_threshold) {
	print "deny from $_ \"# $count{$_} spam access\"\n";
	$out1 += $count{$_};
	$out2++;
    } else {
# less than threshold needs to be recorded for later use
	print "#deny from $_ \"# $count{$_} spam access\"\n";
    }
}

# Reporting output
print STDERR "$skipped lines skipped\n";
print STDERR "$spams spam recognised out of $lines requests\n";
print STDERR "$out2 spammers, $out1 spam counts are written in the reject list\n";
print STDERR "last updated $time\n";

